var React = require('react');
var tweenState = require('react-tween-state');
var objectAssign = require('object-assign');
var Constants = require('./constants');
var Styles = require('./styles');

var NotificationItem = React.createClass({displayName: "NotificationItem",

  mixins: [tweenState.Mixin],

  propTypes: {
    notification: React.PropTypes.object,
    onRemove: React.PropTypes.func
  },

  getDefaultProps: function() {
    return {
      onRemove: function(uid) {}
    }
  },

  getInitialState: function() {
    // var state = {};
    //
    // if (!this.props.getStyles.overrideStyle) return state;
    //
    // var prop = this._getCssPropertyByPosition();
    //
    // state[prop] = -(Styles.Containers.DefaultStyle.width);
    // state.opacity = 0;
    //
    // return state;
    return {
      visible: false
    }
  },

  componentWillMount: function() {
    var getStyles = this.props.getStyles;
    var level = this.props.notification.level;

    this._styles = {
      notification: getStyles.notification(level),
      title: getStyles.title(level),
      dismiss: getStyles.dismiss(level),
      messageWrapper: getStyles.messageWrapper(level),
      actionWrapper: getStyles.actionWrapper(level),
      action: getStyles.action(level)
    };

    if (!this.props.notification.dismissible) {
      this._styles.notification.cursor = 'default';
    }
  },

  _styles: {},

  _notificationTimeout: null,

  _getCssPropertyByPosition: function() {
    var position = this.props.notification.position;
    var cssProperty;

    switch (position) {
      case Constants.positions.tl:
      case Constants.positions.bl:
        cssProperty = 'left';
        break;

      case Constants.positions.tr:
      case Constants.positions.br:
        cssProperty = 'right';
        break;

      case Constants.positions.tc:
        cssProperty = 'top';
        break;

      case Constants.positions.bc:
        cssProperty = 'bottom';
        break;
    }

    return cssProperty;
  },

  _defaultAction: function(event) {
    event.preventDefault();
    var notification = this.props.notification;
    this._hideNotification();
    notification.action.callback();
  },

  _hideNotification: function() {
    var self = this;
    var notification = this.props.notification;
    var property = this._getCssPropertyByPosition();

    if (this._notificationTimeout) {
      clearTimeout(this._notificationTimeout);
    }

    this.tweenState(property, {
      easing: tweenState.easingTypes.easeInOut,
      duration: Constants.animations.notificationItem.hide,
      endValue: -(Styles.Containers.DefaultStyle.width),
      onEnd: function() {
        self.props.onRemove(notification.uid);
      }
    });
  },

  _dismiss: function() {
    if (!this.props.notification.dismissible) {
      return;
    }

    this._hideNotification();
  },

  _showNotification: function() {
    var property = this._getCssPropertyByPosition();

    // this.tweenState('opacity', {
    //   easing: tweenState.easingTypes.easeInOut,
    //   duration: Constants.animations.notificationItem.show,
    //   endValue: 1
    // });
    //
    // this.tweenState(property, {
    //   easing: tweenState.easingTypes.easeInOut,
    //   duration: Constants.animations.notificationItem.show,
    //   endValue: 0
    // });


  },

  componentDidMount: function() {

    this.setState({
      visible: true
    });

    var self = this;
    var element = React.findDOMNode(this)
		var transitionEvent = whichTransitionEvent();

    element.addEventListener(transitionEvent, function() {
      console.log('transitionEnd');
			// if(self.state.remove) {
      //   self.props.onRemove(self.props.notification.uid);
			// }
		});

    var self = this;
    var notification = this.props.notification;

    if (notification.autoDismiss) {
      this._notificationTimeout = setTimeout(function(){
        self.setState({
          visible: false
        });
        // self._hideNotification();
      }, notification.autoDismiss * 1000);
    }

    this._showNotification();

  },

  render: function() {
    var self = this;
    var notification = this.props.notification;

    // if (this.props.getStyles.overrideStyle) {
    //   var property = this._getCssPropertyByPosition();
    //   this._styles.notification[property] = this.getTweeningValue(property);
    //   this._styles.notification.opacity = this.getTweeningValue('opacity');
    // }

    var className = 'notifications notification-' + notification.level;

    if (this.state.visible) {
      className = className + ' notification-visible';
    } else {
      className = className + ' notification-hidden';
    }

    var dismiss = null;
    var actionButton = null;
    var title = null;
    var message = null;

    if (notification.title) {
      title = React.createElement("h4", {style: this._styles.title}, notification.title)
    }

    if (notification.message) {
      message = (
        React.createElement("div", {style: this._styles.messageWrapper}, 
          notification.message
        )
      );
    }

    if (notification.dismissible) {
      dismiss = React.createElement("span", {className: "notification-close", style: this._styles.dismiss}, "×");
    }

    if (notification.action) {
      actionButton = (
        React.createElement("div", {className: "notification-action-wrapper", style: this._styles.actionWrapper}, 
          React.createElement("button", {className: "notification-action-button", onClick: this._defaultAction, style: this._styles.action}, notification.action.label)
        )
      );
    }

    return (
      React.createElement("div", {className: className, onClick: this._dismiss, style: this._styles.notification}, 
        title, 
        message, 
        dismiss, 
        actionButton
      )
    );
  }

});

/* From Modernizr */
function whichTransitionEvent(){
    var t;
    var el = document.createElement('fakeelement');
    var transitions = {
      'transition':'transitionend',
      'OTransition':'oTransitionEnd',
      'MozTransition':'transitionend',
      'WebkitTransition':'webkitTransitionEnd'
    }

    for(t in transitions){
        if( el.style[t] !== undefined ){
            return transitions[t];
        }
    }
}


module.exports = NotificationItem;
